# -*- coding: utf-8 -*-
import unittest
import sys
sys.path.append("../")
import presentation_line as presentation

class TestPurchaseSalesPresentationLine(unittest.TestCase):

    def _create_cabecera_line_test(self):
        
        self.cabecera_line.cuit = 30709653543
        self.cabecera_line.periodo = 201611
        self.cabecera_line.secuencia = 00
        self.cabecera_line.sinMovimiento = 'S'
        self.cabecera_line.prorratearCFC = 'N'
        self.cabecera_line.cFCGlobal = 'S'
        self.cabecera_line.importeCFCG = 100000001251200
        self.cabecera_line.importeCFCAD = 100000001251200
        self.cabecera_line.importeCFCP = 100000001251200
        self.cabecera_line.importeCFnCG = 100000001251200
        self.cabecera_line.cFCSSyOC = 100000001251200
        self.cabecera_line.cFCCSSyOC = 100000001251200

    def _create_ventas_cbte_line_test(self):
        
        self.ventas_cbte_line.fecha = 20151030
        self.ventas_cbte_line.tipo = '091'
        self.ventas_cbte_line.puntoDeVenta = 00001
        self.ventas_cbte_line.numeroComprobante = 4590
        self.ventas_cbte_line.numeroHasta = 4600
        self.ventas_cbte_line.codigoDocumento = 20
        self.ventas_cbte_line.numeroComprador = 51231
        self.ventas_cbte_line.denominacionComprador = 'Test Test'
        self.ventas_cbte_line.importeTotal = 5800
        self.ventas_cbte_line.importeTotalNG = 5000
        self.ventas_cbte_line.percepcionNC = 300
        self.ventas_cbte_line.importeExentos = 123   
        self.ventas_cbte_line.importePercepciones = 5123
        self.ventas_cbte_line.importePerIIBB = 3331
        self.ventas_cbte_line.importePerIM = 4234
        self.ventas_cbte_line.importeImpInt = '12312'
        self.ventas_cbte_line.codigoMoneda = '321'
        self.ventas_cbte_line.tipoCambio = '4444000000'
        self.ventas_cbte_line.cantidadAlicIva = 1
        self.ventas_cbte_line.codigoOperacion = 1
        self.ventas_cbte_line.otrosTributos = 0
        self.ventas_cbte_line.fechaVtoPago = 20151130
    
    def _create_ventas_alicuotas_line_test(self):
        
        self.ventas_alicuotas_line.tipoComprobante = 91
        self.ventas_alicuotas_line.puntoDeVenta = 00001
        self.ventas_alicuotas_line.numeroComprobante = 9898989898
        self.ventas_alicuotas_line.importeNetoGravado = 10000
        self.ventas_alicuotas_line.alicuotaIva = 21
        self.ventas_alicuotas_line.impuestoLiquidado = 2100
 
    def _create_compras_cbte_line_test(self):
        
        self.compras_cbte_line.fecha = 20161212
        self.compras_cbte_line.tipo = 98
        self.compras_cbte_line.puntoDeVenta = 0001
        self.compras_cbte_line.numeroComprobante = '12345678'
        self.compras_cbte_line.despachoImportacion = 'despaimportacion'
        self.compras_cbte_line.codigoDocumento = 'CD'
        self.compras_cbte_line.numeroVendedor = '30709653543'
        self.compras_cbte_line.denominacionVendedor = 'Test test'
        self.compras_cbte_line.importeTotal = 100000
        self.compras_cbte_line.importeTotalNG = 0
        self.compras_cbte_line.importeOpExentas = 0
        self.compras_cbte_line.importePerOIva = 0
        self.compras_cbte_line.importePerOtrosImp = 0
        self.compras_cbte_line.importePerIIBB = 0
        self.compras_cbte_line.importePerIM = 0
        self.compras_cbte_line.importeImpInt = 1000
        self.compras_cbte_line.codigoMoneda = 'ARS'
        self.compras_cbte_line.tipoCambio = '1'
        self.compras_cbte_line.cantidadAlicIva = '5'
        self.compras_cbte_line.codigoOperacion = 'A'
        self.compras_cbte_line.credFiscComp = '0'
        self.compras_cbte_line.otrosTrib = '0'
        self.compras_cbte_line.cuitEmisor = '11022598'
        self.compras_cbte_line.denominacionEmisor = 'Responsable Inscripto'
        self.compras_cbte_line.ivaComision = '100' 
    
    def _create_compras_alicuotas_line_test(self):
        
        self.compras_alicuotas_line.tipoComprobante = '093'
        self.compras_alicuotas_line.puntoDeVenta = '00005'
        self.compras_alicuotas_line.numeroComprobante = 123516129872
        self.compras_alicuotas_line.codigoDocVend = 13
        self.compras_alicuotas_line.numeroIdVend = 26288940
        self.compras_alicuotas_line.importeNetoGravado = 50000
        self.compras_alicuotas_line.alicuotaIva = 3
        self.compras_alicuotas_line.impuestoLiquidado = 10000
     
    def _create_compras_importaciones_line_test(self):
        
        self.compras_importaciones_line.despachoImportacion = '18273918273JKL'
        self.compras_importaciones_line.importeNetoGravado = '100000'
        self.compras_importaciones_line.alicuotaIva = '912'
        self.compras_importaciones_line.impuestoLiquidado = '1000'
        
    def _create_credito_fiscal_importacion_serv_line_test(self):
        
        self.credito_fiscal_importacion_serv_line.tipoComprobante = '3'
        self.credito_fiscal_importacion_serv_line.descripcion = 'Descripcion'
        self.credito_fiscal_importacion_serv_line.identificacionComprobante = '981273918CCLL'
        self.credito_fiscal_importacion_serv_line.fechaOperacion = '20161212'
        self.credito_fiscal_importacion_serv_line.montoMonedaOriginal = '10000'
        self.credito_fiscal_importacion_serv_line.codigoMoneda = 'USD'
        self.credito_fiscal_importacion_serv_line.tipoCambio = '15'
        self.credito_fiscal_importacion_serv_line.cuitPrestador = '262841231'
        self.credito_fiscal_importacion_serv_line.nifPrestador = '123123123245927'
        self.credito_fiscal_importacion_serv_line.nombrePrestador = 'Nombre Apellido Prestador'
        self.credito_fiscal_importacion_serv_line.alicuotaAplicable = '2100'
        self.credito_fiscal_importacion_serv_line.fechaIngresoImpuesto = '20141213'
        self.credito_fiscal_importacion_serv_line.montoImpuesto = '1300'
        self.credito_fiscal_importacion_serv_line.impuestoComputable = '1000'
        self.credito_fiscal_importacion_serv_line.idPago = '0001-8192839192'
        self.credito_fiscal_importacion_serv_line.cuitEntidadPago = '30709612352'

    def _get_lines(self):
        
        lines = [self.cabecera_line, self.ventas_cbte_line, self.ventas_alicuotas_line,
                 self.compras_cbte_line, self.compras_alicuotas_line,
                 self.compras_importaciones_line, self.credito_fiscal_importacion_serv_line]
        
        return lines
    
    cabecera_line = presentation.PresentationLine.factory("ventasCompras", "cabecera")
    ventas_cbte_line = presentation.PresentationLine.factory("ventasCompras", "ventasCbte")
    ventas_alicuotas_line = presentation.PresentationLine.factory("ventasCompras", "ventasAlicuotas")
    compras_cbte_line = presentation.PresentationLine.factory("ventasCompras", "comprasCbte")
    compras_alicuotas_line = presentation.PresentationLine.factory("ventasCompras", "comprasAlicuotas")
    compras_importaciones_line = presentation.PresentationLine.factory("ventasCompras", "comprasImportaciones")
    credito_fiscal_importacion_serv_line = presentation.PresentationLine.factory("ventasCompras", "creditoFiscalImportacionServ")
        
    def test_create_presentation_lines(self):
        self._create_cabecera_line_test()
        self._create_ventas_cbte_line_test()
        self._create_ventas_alicuotas_line_test()
        self._create_compras_cbte_line_test()
        self._create_compras_alicuotas_line_test()
        self._create_compras_importaciones_line_test()
        self._create_credito_fiscal_importacion_serv_line_test()
                
    def test_invalid_attribute(self):
        with self.assertRaises(AttributeError):
            self.cabecera_line.otherValue = 'other value'
    
    def test_more_digits(self):
        with self.assertRaises(ValueError):
            self.credito_fiscal_importacion_serv_line.cuitEntidadPago = '307096123522'
    
    def test_caracteres_especiales(self):          
        self.credito_fiscal_importacion_serv_line.codigoMoneda = 'AÑ|'

    def test_generate_line_string(self):
        for line in self._get_lines():
            line.get_line_string()

    def test_check_lines_len(self):

        self._create_cabecera_line_test()        
        self.assertEqual(len(self.cabecera_line.get_line_string()), 112)
        self._create_ventas_cbte_line_test()
        self.assertEqual(len(self.ventas_cbte_line.get_line_string()), 266)
        self._create_ventas_alicuotas_line_test()
        self.assertEqual(len(self.ventas_alicuotas_line.get_line_string()), 62)
        self._create_compras_cbte_line_test()
        self.assertEqual(len(self.compras_cbte_line.get_line_string()), 325)
        self._create_compras_alicuotas_line_test()
        self.assertEqual(len(self.compras_alicuotas_line.get_line_string()), 84)
        self._create_compras_importaciones_line_test()
        self.assertEqual(len(self.compras_importaciones_line.get_line_string()), 50)
        self._create_credito_fiscal_importacion_serv_line_test()
        self.assertEqual(len(self.credito_fiscal_importacion_serv_line.get_line_string()), 211)
        
if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
# Segun RG 2485 – Proyecto FE v2.8 - 12/09/2016
import sys
sys.path.append("..")
from zeep import Client
from .. import config


class WsSrPadron(object):
    """
    Consulta Padron A4.
    
    :param access_token: AccessToken - Token de acceso
    :param cuit: Cuit de la empresa
    :param homologation: Homologacion si es True
    :param url: Url de servicios para WsSrPadron
    """
        
    def __init__(self, access_token, cuit, homologation=True, url=None):
        if not url:
            self.url = config.service_urls.get('ws_sr_padron_a4_homologation') if homologation\
                else config.service_urls.get('ws_sr_padron_a4_production')
        else:
            self.url = url
        
        self.accessToken = access_token
        self.cuit = cuit

    def get_partner_data(self, vat):
        """
        :param vat: Numero de documentos a consultar
        :returns str: Respuesta de AFIP sobre el numero de documento
        """

        response = Client(self.url).service.getPersona(
            token=self.accessToken.token,
            sign=self.accessToken.sign,
            cuitRepresentada=self.cuit,
            idPersona=vat
        )

        return response



authorization_urls = dict(
    homologation = 'https://wsaahomo.afip.gov.ar/ws/services/LoginCms?wsdl',
    production = 'https://wsaa.afip.gov.ar/ws/services/LoginCms?wsdl', 
)

service_urls = dict(
    wsfev1_homologation = 'https://wswhomo.afip.gov.ar/wsfev1/service.asmx?wsdl',
    wsfev1_production = 'https://servicios1.afip.gov.ar/wsfev1/service.asmx?wsdl', 
    wsfexv1_homologation = 'https://wswhomo.afip.gov.ar/wsfexv1/service.asmx?wsdl',
    wsfexv1_production = 'https://servicios1.afip.gov.ar/wsfexv1/service.asmx?wsdl',
    ws_sr_padron_a4_homologation = 'https://awshomo.afip.gov.ar/sr-padron/webservices/personaServiceA4?WSDL',
    ws_sr_padron_a4_production = 'https://aws.afip.gov.ar/sr-padron/webservices/personaServiceA4?WSDL',
)


# -*- coding: utf-8 -*-  
import presentation_line
import zipfile
import StringIO
import time
import base64

class Presentation(object):
          
    def __init__(self, presentation, presentation_type):
        self._lines = []
        self._presentation = presentation
        self._presentation_type = presentation_type

    def get_encoded_string(self):
        '''
        :return: String en base 64
        '''
        return base64.encodestring(self.get_string())   
        
    def get_string(self):
        '''
        :return: String con todas las lineas de esa lista
        '''
        
        presentation_string = '\r\n'.join(line.get_line_string() for line in self.lines)
        return presentation_string
  
    def create_line(self):
        line = presentation_line.PresentationLine.factory(self.presentation, self.presentation_type)
        self.add_line(line)
        return line
        
    @property
    def lines(self):
        return self._lines
    
    def add_line(self, item):
        self.lines.append(item)
        
    @property
    def presentation(self):
        return self._presentation

    @property
    def presentation_type(self):
        return self._presentation_type

class PresentationZipExporter(object):
    
    def __init__(self):  
        #Lista de presentaciones  
        self._presentations_to_export = []

    def export_elements(self):
        '''
        Exporta las presentaciones a un archivo ZIP en base 64
        '''
        zip_buffer = StringIO.StringIO()
        zf = zipfile.ZipFile( zip_buffer, mode='a',compression=zipfile.ZIP_DEFLATED)
        
        for presentation in self.presentations_to_export:
            
            if not presentation.file_name:
                raise AttributeError("La presentacion no tiene nombre para el archivo")
                
            info = zipfile.ZipInfo(presentation.file_name, date_time=time.localtime(time.time()),)
            info.compress_type=zipfile.ZIP_DEFLATED
            info.create_system=0
            zf.writestr(info, presentation.get_string())
        
        zf.close()
        return base64.encodestring(zip_buffer.getvalue())    
                   
    @property
    def presentations_to_export(self):
        return self._presentations_to_export
    
    def add_element(self, presentation):
        self.presentations_to_export.append(presentation)



import pytest
import sys, os
sys.path.append(os.path.realpath(os.path.dirname(__file__)+"/.."))
from contributor import Contributor

class TestContribuyentes():

    def test_valid_cuit(self):    
        cuit = '30709653543'
        assert Contributor.is_valid_cuit(cuit) 
        cuit = 30709653543
        assert Contributor.is_valid_cuit(cuit)

    def test_invalid_cuit(self):
        cuit = '30709653542'
        assert Contributor.is_valid_cuit(cuit) == 0
        cuit = 30709653542
        assert Contributor.is_valid_cuit(cuit) == 0

    def test_cuit_without_11_digits(self):
        cuit = '3070965354'
        assert Contributor.is_valid_cuit(cuit) == 0
        cuit = 3070965354
        assert Contributor.is_valid_cuit(cuit) == 0
                        
    def test_11_digits_no_int(self):  
        cuit = 'A1234567891'
        assert Contributor.is_valid_cuit(cuit) == 0
        
    def test_cuit_with_dash(self):
        cuit = '30-70965354-3'
        assert Contributor.is_valid_cuit(cuit) == 0
    
    def test_get_partner_data(self):
        Contributor.get_partner_data('30709653543')
        
    def test_get_data_invalid_url(self):
        with pytest.raises(ValueError):
            Contributor.get_partner_data('30709653543', url="invalidUrl")
                
    def test_get_data_no_json(self):
        with pytest.raises(ValueError):
            Contributor.get_partner_data('30709653543', url="https://www.google.com.ar/?gfe_rd=cr&ei=uWRqWLaUNYyTwASCjLPABQ&gws_rd=ssl#q=")
        
    def test_set_partner_data(self):
        contributor = Contributor()
        contributor.set_partner_data('30712097953')
        assert contributor.document_number == '30712097953'
        assert contributor.name == 'OPENPYME SRL'
        assert contributor.postal_code == '1425'
        assert contributor.street == 'BILLINGHURST 1964'
        assert contributor.state_code == 0

    def test_set_partner_data_invalid_cuit(self):
        contributor = Contributor()
        contributor.set_partner_data('307120979531')
        assert contributor.document_number == '307120979531'
        assert contributor.name == None
        assert contributor.postal_code == None
        assert contributor.street == None
        assert contributor.state_code == None


# -*- coding: utf-8 -*-

class PresentationLine(object):
    __slots__ = []

    @staticmethod
    def factory(presentation, line_name):
        '''
        :param presentation: Tipo de presentacion
        :param line_name: Nombre de la linea a completar para la presentacion
        '''

        if presentation == "ventasCompras":

            if line_name == "cabecera": return PurchaseSalesPresentationCabeceraLine()
            if line_name == "ventasCbte": return PurchaseSalesPresentationVentasCbteLine()
            if line_name == "ventasAlicuotas": return PurchaseSalesPresentationVentasAlicuotasLine()
            if line_name == "comprasCbte": return PurchaseSalesPresentationComprasCbteLine()
            if line_name == "comprasAlicuotas": return PurchaseSalesPresentationComprasAlicuotasLine()
            if line_name == "comprasImportaciones": return PurchaseSalesPresentationComprasImportacionesLine()
            if line_name == "creditoFiscalImportacionServ": return PurchaseSalesPresentationCreditoFiscalImportacionServLine()

        if presentation == "sifere":

            if line_name == "retenciones": return SifereRetentionLine()
            if line_name == "percepciones": return SiferePerceptionLine()

        if presentation == "sicore":

            if line_name == "retenciones": return SicoreRetentionLine()

        assert 0, "No existe la presentacion: " + presentation + ", o el tipo: " + line_name

    def _fill_and_validate_len(self, attribute, variable, length, numeric=True):
        '''
        :param attribute: Atributo a validar la longitud
        :param length: Longitud que deberia tener
        '''

        attribute = self._convert_to_string(attribute, variable)
        attribute = attribute.zfill(length) if numeric else attribute.ljust(length)[:length]

        if len(attribute) > length and len(attribute.decode('utf-8')) > length:
            raise ValueError(('El valor {variable} contiene mas digitos de '
                              'los pre-establecidos').format(variable=variable))

        return attribute

    def _convert_to_string(self, attribute, variable):
        '''
        :param attribute: Atributo para pasar a str
        '''

        try:
            attribute = str(attribute)
        except ValueError:
            raise ValueError('Valor {variable} erroneo o incompleto'.format(variable=variable))

        return attribute

    def get_line_string(self):

        try:
            line_string = ''.join(self.get_values())
        except TypeError:
            raise TypeError("La linea esta incompleta o es erronea")

        return line_string

    def get_values(self):
        raise NotImplementedError("Funcion get_values no implementada para esta clase")


class PurchaseSalesPresentationCabeceraLine(PresentationLine):
    __slots__ = ['_cuit', '_periodo', '_secuencia', '_sinMovimiento',
                 '_prorratearCFC', '_cFCGlobal', '_importeCFCG', '_importeCFCAD',
                 '_importeCFCP', '_importeCFnCG', '_cFCSSyOC', '_cFCCSSyOC']

    def __init__(self):
        self._cuit = None
        self._periodo = None
        self._secuencia = None
        self._sinMovimiento = None
        self._prorratearCFC = None
        self._cFCGlobal = None
        self._importeCFCG = None
        self._importeCFCAD = None
        self._importeCFCP = None
        self._importeCFnCG = None
        self._cFCSSyOC = None
        self._cFCCSSyOC = None

    @property
    def cuit(self):
        return self._cuit

    @cuit.setter
    def cuit(self, cuit):
        self._cuit = self._fill_and_validate_len(cuit, 'cuit', 11)

    @property
    def periodo(self):
        return self._periodo

    @periodo.setter
    def periodo(self, periodo):
        self._periodo = self._fill_and_validate_len(periodo, 'periodo', 6)

    @property
    def secuencia(self):
        return self._secuencia

    @secuencia.setter
    def secuencia(self, secuencia):
        self._secuencia = self._fill_and_validate_len(secuencia, 'secuencia', 2)

    @property
    def sinMovimiento(self):
        return self._sinMovimiento

    @sinMovimiento.setter
    def sinMovimiento(self, sinMovimiento):
        self._sinMovimiento = self._fill_and_validate_len(sinMovimiento, 'sinMovimiento', 1, numeric=False)

    @property
    def prorratearCFC(self):
        return self._prorratearCFC

    @prorratearCFC.setter
    def prorratearCFC(self, prorratearCFC):
        self._prorratearCFC = self._fill_and_validate_len(prorratearCFC, 'prorratearCFC', 1, numeric=False)

    @property
    def cFCGlobal(self):
        return self._cFCGlobal

    @cFCGlobal.setter
    def cFCGlobal(self, cFCGlobal):
        self._cFCGlobal = self._fill_and_validate_len(cFCGlobal, 'cFCGlobal', 1, numeric=False)

    @property
    def importeCFCG(self):
        return self._importeCFCG

    @importeCFCG.setter
    def importeCFCG(self, importeCFCG):
        self._importeCFCG = self._fill_and_validate_len(importeCFCG, 'importeCFCG', 15)

    @property
    def importeCFCAD(self):
        return self._importeCFCAD

    @importeCFCAD.setter
    def importeCFCAD(self, importeCFCAD):
        self._importeCFCAD = self._fill_and_validate_len(importeCFCAD, 'importeCFCAD', 15)

    @property
    def importeCFCP(self):
        return self._importeCFCP

    @importeCFCP.setter
    def importeCFCP(self, importeCFCP):
        self._importeCFCP = self._fill_and_validate_len(importeCFCP, 'importeCFCP', 15)

    @property
    def importeCFnCG(self):
        return self._importeCFnCG

    @importeCFnCG.setter
    def importeCFnCG(self, importeCFnCG):
        self._importeCFnCG = self._fill_and_validate_len(importeCFnCG, 'importeCFnCG', 15)

    @property
    def cFCSSyOC(self):
        return self._cFCSSyOC

    @cFCSSyOC.setter
    def cFCSSyOC(self, cFCSSyOC):
        self._cFCSSyOC = self._fill_and_validate_len(cFCSSyOC, 'cFCSSyOC', 15)

    @property
    def cFCCSSyOC(self):
        return self._cFCCSSyOC

    @cFCCSSyOC.setter
    def cFCCSSyOC(self, cFCCSSyOC):
        self._cFCCSSyOC = self._fill_and_validate_len(cFCCSSyOC, 'cFCCSSyOC', 15)

    def get_values(self):
        values = [self.cuit, self.periodo, self.secuencia, self.sinMovimiento,
                  self.prorratearCFC, self.cFCGlobal, self.importeCFCG,
                  self.importeCFCAD, self.importeCFCP, self.importeCFnCG,
                  self.cFCSSyOC, self.cFCCSSyOC]

        return values


class PurchaseSalesPresentationVentasCbteLine(PresentationLine):
    __slots__ = ['_fecha', '_tipo', '_puntoDeVenta', '_numeroComprobante', '_numeroHasta',
                 '_codigoDocumento', '_numeroComprador', '_denominacionComprador',
                 '_importeTotal', '_importeTotalNG', '_percepcionNC', '_importeExentos',
                 '_importePercepciones', '_importePerIIBB', '_importePerIM',
                 '_importeImpInt', '_codigoMoneda', '_tipoCambio', '_cantidadAlicIva',
                 '_codigoOperacion', '_otrosTributos', '_fechaVtoPago']

    def __init__(self):
        self._fecha = None
        self._tipo = None
        self._puntoDeVenta = None
        self._numeroComprobante = None
        self._numeroHasta = None
        self._codigoDocumento = None
        self._numeroComprador = None
        self._denominacionComprador = None
        self._importeTotal = None
        self._importeTotalNG = None
        self._percepcionNC = None
        self._importeExentos = None
        self._importePercepciones = None
        self._importePerIIBB = None
        self._importePerIM = None
        self._importeImpInt = None
        self._codigoMoneda = None
        self._tipoCambio = None
        self._cantidadAlicIva = None
        self._codigoOperacion = None
        self._otrosTributos = None
        self._fechaVtoPago = None

    def get_values(self):
        values = [self.fecha, self.tipo, self.puntoDeVenta, self.numeroComprobante,
                  self.numeroHasta, self.codigoDocumento, self.numeroComprador,
                  self.denominacionComprador, self.importeTotal, self.importeTotalNG,
                  self.percepcionNC, self.importeExentos, self.importePercepciones,
                  self.importePerIIBB, self.importePerIM, self.importeImpInt,
                  self.codigoMoneda, self.tipoCambio, self.cantidadAlicIva,
                  self.codigoOperacion, self.otrosTributos, self.fechaVtoPago]

        return values

    @property
    def fecha(self):
        return self._fecha

    @fecha.setter
    def fecha(self, fecha):
        self._fecha = self._fill_and_validate_len(fecha, 'fecha', 8)

    @property
    def tipo(self):
        return self._tipo

    @tipo.setter
    def tipo(self, tipo):
        self._tipo = self._fill_and_validate_len(tipo, 'tipo', 3)

    @property
    def puntoDeVenta(self):
        return self._puntoDeVenta

    @puntoDeVenta.setter
    def puntoDeVenta(self, puntoDeVenta):
        self._puntoDeVenta = self._fill_and_validate_len(puntoDeVenta, 'puntoDeVenta', 5)

    @property
    def numeroComprobante(self):
        return self._numeroComprobante

    @numeroComprobante.setter
    def numeroComprobante(self, numeroComprobante):
        self._numeroComprobante = self._fill_and_validate_len(numeroComprobante, 'numeroComprobante', 20)

    @property
    def numeroHasta(self):
        return self._numeroHasta

    @numeroHasta.setter
    def numeroHasta(self, numeroHasta):
        self._numeroHasta = self._fill_and_validate_len(numeroHasta, 'numeroHasta', 20)

    @property
    def codigoDocumento(self):
        return self._codigoDocumento

    @codigoDocumento.setter
    def codigoDocumento(self, codigoDocumento):
        self._codigoDocumento = self._fill_and_validate_len(codigoDocumento, 'codigoDocumento', 2)

    @property
    def numeroComprador(self):
        return self._numeroComprador

    @numeroComprador.setter
    def numeroComprador(self, numeroComprador):
        self._numeroComprador = self._fill_and_validate_len(numeroComprador, 'numeroComprador', 20)

    @property
    def denominacionComprador(self):
        return self._denominacionComprador

    @denominacionComprador.setter
    def denominacionComprador(self, denominacionComprador):
        self._denominacionComprador = self._fill_and_validate_len(denominacionComprador, 'denominacionComprador', 30,
                                                                  numeric=False)

    @property
    def importeTotal(self):
        return self._importeTotal

    @importeTotal.setter
    def importeTotal(self, importeTotal):
        self._importeTotal = self._fill_and_validate_len(importeTotal, 'importeTotal', 15)

    @property
    def importeTotalNG(self):
        return self._importeTotalNG

    @importeTotalNG.setter
    def importeTotalNG(self, importeTotalNG):
        self._importeTotalNG = self._fill_and_validate_len(importeTotalNG, 'importeTotalNG', 15)

    @property
    def percepcionNC(self):
        return self._percepcionNC

    @percepcionNC.setter
    def percepcionNC(self, percepcionNC):
        self._percepcionNC = self._fill_and_validate_len(percepcionNC, 'percepcionNC', 15)

    @property
    def importeExentos(self):
        return self._importeExentos

    @importeExentos.setter
    def importeExentos(self, importeExentos):
        self._importeExentos = self._fill_and_validate_len(importeExentos, 'importeExentos', 15)

    @property
    def importePercepciones(self):
        return self._importePercepciones

    @importePercepciones.setter
    def importePercepciones(self, importePercepciones):
        self._importePercepciones = self._fill_and_validate_len(importePercepciones, 'importePercepciones', 15)

    @property
    def importePerIIBB(self):
        return self._importePerIIBB

    @importePerIIBB.setter
    def importePerIIBB(self, importePerIIBB):
        self._importePerIIBB = self._fill_and_validate_len(importePerIIBB, 'importePerIIBB', 15)

    @property
    def importePerIM(self):
        return self._importePerIM

    @importePerIM.setter
    def importePerIM(self, importePerIM):
        self._importePerIM = self._fill_and_validate_len(importePerIM, 'importePerIM', 15)

    @property
    def importeImpInt(self):
        return self._importeImpInt

    @importeImpInt.setter
    def importeImpInt(self, importeImpInt):
        self._importeImpInt = self._fill_and_validate_len(importeImpInt, 'importeImpInt', 15)

    @property
    def codigoMoneda(self):
        return self._codigoMoneda

    @codigoMoneda.setter
    def codigoMoneda(self, codigoMoneda):
        self._codigoMoneda = self._fill_and_validate_len(codigoMoneda, 'codigoMoneda', 3)

    @property
    def tipoCambio(self):
        return self._tipoCambio

    @tipoCambio.setter
    def tipoCambio(self, tipoCambio):
        self._tipoCambio = self._fill_and_validate_len(tipoCambio, 'tipoCambio', 10)

    @property
    def cantidadAlicIva(self):
        return self._cantidadAlicIva

    @cantidadAlicIva.setter
    def cantidadAlicIva(self, cantidadAlicIva):
        self._cantidadAlicIva = self._fill_and_validate_len(cantidadAlicIva, 'cantidadAlicIva', 1)

    @property
    def codigoOperacion(self):
        return self._codigoOperacion

    @codigoOperacion.setter
    def codigoOperacion(self, codigoOperacion):
        self._codigoOperacion = self._fill_and_validate_len(codigoOperacion, 'codigoOperacion', 1, numeric=False)

    @property
    def otrosTributos(self):
        return self._otrosTributos

    @otrosTributos.setter
    def otrosTributos(self, otrosTributos):
        self._otrosTributos = self._fill_and_validate_len(otrosTributos, 'otrosTributos', 15)

    @property
    def fechaVtoPago(self):
        return self._fechaVtoPago

    @fechaVtoPago.setter
    def fechaVtoPago(self, fechaVtoPago):
        self._fechaVtoPago = self._fill_and_validate_len(fechaVtoPago, 'fechaVtoPago', 8)


class PurchaseSalesPresentationVentasAlicuotasLine(PresentationLine):
    __slots__ = ['_tipoComprobante', '_puntoDeVenta', '_numeroComprobante',
                 '_importeNetoGravado', '_alicuotaIva', '_impuestoLiquidado']

    def __init__(self):
        self._tipoComprobante = None
        self._puntoDeVenta = None
        self._numeroComprobante = None
        self._importeNetoGravado = None
        self._alicuotaIva = None
        self._impuestoLiquidado = None

    def get_values(self):
        values = [self.tipoComprobante, self.puntoDeVenta, self.numeroComprobante,
                  self.importeNetoGravado, self.alicuotaIva, self.impuestoLiquidado]

        return values

    @property
    def tipoComprobante(self):
        return self._tipoComprobante

    @tipoComprobante.setter
    def tipoComprobante(self, tipoComprobante):
        self._tipoComprobante = self._fill_and_validate_len(tipoComprobante, 'tipoComprobante', 3)

    @property
    def puntoDeVenta(self):
        return self._puntoDeVenta

    @puntoDeVenta.setter
    def puntoDeVenta(self, puntoDeVenta):
        self._puntoDeVenta = self._fill_and_validate_len(puntoDeVenta, 'puntoDeVenta', 5)

    @property
    def numeroComprobante(self):
        return self._numeroComprobante

    @numeroComprobante.setter
    def numeroComprobante(self, numeroComprobante):
        self._numeroComprobante = self._fill_and_validate_len(numeroComprobante, 'numeroComprobante', 20)

    @property
    def importeNetoGravado(self):
        return self._importeNetoGravado

    @importeNetoGravado.setter
    def importeNetoGravado(self, importeNetoGravado):
        self._importeNetoGravado = self._fill_and_validate_len(importeNetoGravado, 'importeNetoGravado', 15)

    @property
    def alicuotaIva(self):
        return self._alicuotaIva

    @alicuotaIva.setter
    def alicuotaIva(self, alicuotaIva):
        self._alicuotaIva = self._fill_and_validate_len(alicuotaIva, 'alicuotaIva', 4)

    @property
    def impuestoLiquidado(self):
        return self._impuestoLiquidado

    @impuestoLiquidado.setter
    def impuestoLiquidado(self, impuestoLiquidado):
        self._impuestoLiquidado = self._fill_and_validate_len(impuestoLiquidado, 'impuestoLiquidado', 15)


class PurchaseSalesPresentationComprasCbteLine(PresentationLine):
    __slots__ = ['_fecha', '_tipo', '_puntoDeVenta', '_numeroComprobante',
                 '_despachoImportacion', '_codigoDocumento', '_numeroVendedor',
                 '_denominacionVendedor', '_importeTotal', '_importeTotalNG',
                 '_importeOpExentas', '_importePerOIva', '_importePerOtrosImp',
                 '_importePerIIBB', '_importePerIM', '_importeImpInt',
                 '_codigoMoneda', '_tipoCambio', '_cantidadAlicIva',
                 '_codigoOperacion', '_credFiscComp', '_otrosTrib',
                 '_cuitEmisor', '_denominacionEmisor', '_ivaComision']

    def __init__(self):
        self._fecha = None
        self._tipo = None
        self._puntoDeVenta = None
        self._numeroComprobante = None
        self._despachoImportacion = None
        self._codigoDocumento = None
        self._numeroVendedor = None
        self._denominacionVendedor = None
        self._importeTotal = None
        self._importeTotalNG = None
        self._importeOpExentas = None
        self._importePerOIva = None
        self._importePerOtrosImp = None
        self._importePerIIBB = None
        self._importePerIM = None
        self._importeImpInt = None
        self._codigoMoneda = None
        self._tipoCambio = None
        self._cantidadAlicIva = None
        self._codigoOperacion = None
        self._credFiscComp = None
        self._otrosTrib = None
        self._cuitEmisor = None
        self._denominacionEmisor = None
        self._ivaComision = None

    def get_values(self):
        values = [self.fecha, self.tipo, self.puntoDeVenta, self.numeroComprobante,
                  self.despachoImportacion, self.codigoDocumento, self.numeroVendedor,
                  self.denominacionVendedor, self.importeTotal, self.importeTotalNG,
                  self.importeOpExentas, self.importePerOIva, self.importePerOtrosImp,
                  self.importePerIIBB, self.importePerIM, self.importeImpInt,
                  self.codigoMoneda, self.tipoCambio, self.cantidadAlicIva,
                  self.codigoOperacion, self.credFiscComp, self.otrosTrib,
                  self.cuitEmisor, self.denominacionEmisor, self.ivaComision]

        return values

    @property
    def fecha(self):
        return self._fecha

    @fecha.setter
    def fecha(self, fecha):
        self._fecha = self._fill_and_validate_len(fecha, 'fecha', 8)

    @property
    def tipo(self):
        return self._tipo

    @tipo.setter
    def tipo(self, tipo):
        self._tipo = self._fill_and_validate_len(tipo, 'tipo', 3)

    @property
    def puntoDeVenta(self):
        return self._puntoDeVenta

    @puntoDeVenta.setter
    def puntoDeVenta(self, puntoDeVenta):
        self._puntoDeVenta = self._fill_and_validate_len(puntoDeVenta, 'puntoDeVenta', 5)

    @property
    def numeroComprobante(self):
        return self._numeroComprobante

    @numeroComprobante.setter
    def numeroComprobante(self, numeroComprobante):
        self._numeroComprobante = self._fill_and_validate_len(numeroComprobante, 'numeroComprobante', 20)

    @property
    def despachoImportacion(self):
        return self._despachoImportacion

    @despachoImportacion.setter
    def despachoImportacion(self, despachoImportacion):
        self._despachoImportacion = self._fill_and_validate_len(despachoImportacion, 'despachoImportacion', 16,
                                                                numeric=False)

    @property
    def codigoDocumento(self):
        return self._codigoDocumento

    @codigoDocumento.setter
    def codigoDocumento(self, codigoDocumento):
        self._codigoDocumento = self._fill_and_validate_len(codigoDocumento, 'codigoDocumento', 2)

    @property
    def numeroVendedor(self):
        return self._numeroVendedor

    @numeroVendedor.setter
    def numeroVendedor(self, numeroVendedor):
        self._numeroVendedor = self._fill_and_validate_len(numeroVendedor, 'numeroVendedor', 20)

    @property
    def denominacionVendedor(self):
        return self._denominacionVendedor

    @denominacionVendedor.setter
    def denominacionVendedor(self, denominacionVendedor):
        self._denominacionVendedor = self._fill_and_validate_len(denominacionVendedor, 'denominacionVendedor', 30,
                                                                 numeric=False)

    @property
    def importeTotal(self):
        return self._importeTotal

    @importeTotal.setter
    def importeTotal(self, importeTotal):
        self._importeTotal = self._fill_and_validate_len(importeTotal, 'importeTotal', 15)

    @property
    def importeTotalNG(self):
        return self._importeTotalNG

    @importeTotalNG.setter
    def importeTotalNG(self, importeTotalNG):
        self._importeTotalNG = self._fill_and_validate_len(importeTotalNG, 'importeTotalNG', 15)

    @property
    def importeOpExentas(self):
        return self._importeOpExentas

    @importeOpExentas.setter
    def importeOpExentas(self, importeOpExentas):
        self._importeOpExentas = self._fill_and_validate_len(importeOpExentas, 'importeOpExentas', 15)

    @property
    def importePerOIva(self):
        return self._importePerOIva

    @importePerOIva.setter
    def importePerOIva(self, importePerOIva):
        self._importePerOIva = self._fill_and_validate_len(importePerOIva, 'importePerOIva', 15)

    @property
    def importePerOtrosImp(self):
        return self._importePerOtrosImp

    @importePerOtrosImp.setter
    def importePerOtrosImp(self, importePerOtrosImp):
        self._importePerOtrosImp = self._fill_and_validate_len(importePerOtrosImp, 'importePerOtrosImp', 15)

    @property
    def importePerIIBB(self):
        return self._importePerIIBB

    @importePerIIBB.setter
    def importePerIIBB(self, importePerIIBB):
        self._importePerIIBB = self._fill_and_validate_len(importePerIIBB, 'importePerIIBB', 15)

    @property
    def importePerIM(self):
        return self._importePerIM

    @importePerIM.setter
    def importePerIM(self, importePerIM):
        self._importePerIM = self._fill_and_validate_len(importePerIM, 'importePerIM', 15)

    @property
    def importeImpInt(self):
        return self._importeImpInt

    @importeImpInt.setter
    def importeImpInt(self, importeImpInt):
        self._importeImpInt = self._fill_and_validate_len(importeImpInt, 'importeImpInt', 15)

    @property
    def codigoMoneda(self):
        return self._codigoMoneda

    @codigoMoneda.setter
    def codigoMoneda(self, codigoMoneda):
        self._codigoMoneda = self._fill_and_validate_len(codigoMoneda, 'codigoMoneda', 3, numeric=False)

    @property
    def tipoCambio(self):
        return self._tipoCambio

    @tipoCambio.setter
    def tipoCambio(self, tipoCambio):
        self._tipoCambio = self._fill_and_validate_len(tipoCambio, 'tipoCambio', 10)

    @property
    def cantidadAlicIva(self):
        return self._cantidadAlicIva

    @cantidadAlicIva.setter
    def cantidadAlicIva(self, cantidadAlicIva):
        self._cantidadAlicIva = self._fill_and_validate_len(cantidadAlicIva, 'cantidadAlicIva', 1)

    @property
    def codigoOperacion(self):
        return self._codigoOperacion

    @codigoOperacion.setter
    def codigoOperacion(self, codigoOperacion):
        self._codigoOperacion = self._fill_and_validate_len(codigoOperacion, 'codigoOperacion', 1, numeric=False)

    @property
    def credFiscComp(self):
        return self._credFiscComp

    @credFiscComp.setter
    def credFiscComp(self, credFiscComp):
        self._credFiscComp = self._fill_and_validate_len(credFiscComp, 'credFiscComp', 15)

    @property
    def otrosTrib(self):
        return self._otrosTrib

    @otrosTrib.setter
    def otrosTrib(self, otrosTrib):
        self._otrosTrib = self._fill_and_validate_len(otrosTrib, 'otrosTrib', 15)

    @property
    def cuitEmisor(self):
        return self._cuitEmisor

    @cuitEmisor.setter
    def cuitEmisor(self, cuitEmisor):
        self._cuitEmisor = self._fill_and_validate_len(cuitEmisor, 'cuitEmisor', 11)

    @property
    def denominacionEmisor(self):
        return self._denominacionEmisor

    @denominacionEmisor.setter
    def denominacionEmisor(self, denominacionEmisor):
        self._denominacionEmisor = self._fill_and_validate_len(denominacionEmisor, 'denominacionEmisor', 30,
                                                               numeric=False)

    @property
    def ivaComision(self):
        return self._ivaComision

    @ivaComision.setter
    def ivaComision(self, ivaComision):
        self._ivaComision = self._fill_and_validate_len(ivaComision, 'ivaComision', 15)


class PurchaseSalesPresentationComprasAlicuotasLine(PresentationLine):
    __slots__ = ['_tipoComprobante', '_puntoDeVenta', '_numeroComprobante',
                 '_codigoDocVend', '_numeroIdVend', '_importeNetoGravado',
                 '_alicuotaIva', '_impuestoLiquidado']

    def __init__(self):
        self._tipoComprobante = None
        self._puntoDeVenta = None
        self._numeroComprobante = None
        self._codigoDocVend = None
        self._numeroIdVend = None
        self._importeNetoGravado = None
        self._alicuotaIva = None
        self._impuestoLiquidado = None

    def get_values(self):
        values = [self.tipoComprobante, self.puntoDeVenta, self.numeroComprobante,
                  self.codigoDocVend, self.numeroIdVend, self.importeNetoGravado,
                  self.alicuotaIva, self.impuestoLiquidado]

        return values

    @property
    def tipoComprobante(self):
        return self._tipoComprobante

    @tipoComprobante.setter
    def tipoComprobante(self, tipoComprobante):
        self._tipoComprobante = self._fill_and_validate_len(tipoComprobante, 'tipoComprobante', 3)

    @property
    def puntoDeVenta(self):
        return self._puntoDeVenta

    @puntoDeVenta.setter
    def puntoDeVenta(self, puntoDeVenta):
        self._puntoDeVenta = self._fill_and_validate_len(puntoDeVenta, 'puntoDeVenta', 5)

    @property
    def numeroComprobante(self):
        return self._numeroComprobante

    @numeroComprobante.setter
    def numeroComprobante(self, numeroComprobante):
        self._numeroComprobante = self._fill_and_validate_len(numeroComprobante, 'numeroComprobante', 20)

    @property
    def codigoDocVend(self):
        return self._codigoDocVend

    @codigoDocVend.setter
    def codigoDocVend(self, codigoDocVend):
        self._codigoDocVend = self._fill_and_validate_len(codigoDocVend, 'codigoDocVend', 2)

    @property
    def numeroIdVend(self):
        return self._numeroIdVend

    @numeroIdVend.setter
    def numeroIdVend(self, numeroIdVend):
        self._numeroIdVend = self._fill_and_validate_len(numeroIdVend, 'numeroIdVend', 20)

    @property
    def importeNetoGravado(self):
        return self._importeNetoGravado

    @importeNetoGravado.setter
    def importeNetoGravado(self, importeNetoGravado):
        self._importeNetoGravado = self._fill_and_validate_len(importeNetoGravado, 'importeNetoGravado', 15)

    @property
    def alicuotaIva(self):
        return self._alicuotaIva

    @alicuotaIva.setter
    def alicuotaIva(self, alicuotaIva):
        self._alicuotaIva = self._fill_and_validate_len(alicuotaIva, 'alicuotaIva', 4)

    @property
    def impuestoLiquidado(self):
        return self._impuestoLiquidado

    @impuestoLiquidado.setter
    def impuestoLiquidado(self, impuestoLiquidado):
        self._impuestoLiquidado = self._fill_and_validate_len(impuestoLiquidado, 'impuestoLiquidado', 15)


class PurchaseSalesPresentationComprasImportacionesLine(PresentationLine):
    __slots__ = ['_despachoImportacion', '_importeNetoGravado',
                 '_alicuotaIva', '_impuestoLiquidado']

    def __init__(self):
        self._despachoImportacion = None
        self._importeNetoGravado = None
        self._alicuotaIva = None
        self._impuestoLiquidado = None

    def get_values(self):
        values = [self.despachoImportacion, self.importeNetoGravado,
                  self.alicuotaIva, self.impuestoLiquidado]

        return values

    @property
    def despachoImportacion(self):
        return self._despachoImportacion

    @despachoImportacion.setter
    def despachoImportacion(self, despachoImportacion):
        self._despachoImportacion = self._fill_and_validate_len(despachoImportacion, 'despachoImportacion', 16,
                                                                numeric=False)

    @property
    def importeNetoGravado(self):
        return self._importeNetoGravado

    @importeNetoGravado.setter
    def importeNetoGravado(self, importeNetoGravado):
        self._importeNetoGravado = self._fill_and_validate_len(importeNetoGravado, 'importeNetoGravado', 15)

    @property
    def alicuotaIva(self):
        return self._alicuotaIva

    @alicuotaIva.setter
    def alicuotaIva(self, alicuotaIva):
        self._alicuotaIva = self._fill_and_validate_len(alicuotaIva, 'alicuotaIva', 4)

    @property
    def impuestoLiquidado(self):
        return self._impuestoLiquidado

    @impuestoLiquidado.setter
    def impuestoLiquidado(self, impuestoLiquidado):
        self._impuestoLiquidado = self._fill_and_validate_len(impuestoLiquidado, 'impuestoLiquidado', 15)


class PurchaseSalesPresentationCreditoFiscalImportacionServLine(PresentationLine):
    __slots__ = ['_tipoComprobante', '_descripcion', '_identificacionComprobante',
                 '_fechaOperacion', '_montoMonedaOriginal', '_codigoMoneda',
                 '_tipoCambio', '_cuitPrestador', '_nifPrestador', '_nombrePrestador',
                 '_alicuotaAplicable', '_fechaIngresoImpuesto', '_montoImpuesto',
                 '_impuestoComputable', '_idPago', '_cuitEntidadPago']

    def __init__(self):
        self._tipoComprobante = None
        self._descripcion = None
        self._identificacionComprobante = None
        self._fechaOperacion = None
        self._montoMonedaOriginal = None
        self._codigoMoneda = None
        self._tipoCambio = None
        self._cuitPrestador = None
        self._nifPrestador = None
        self._nombrePrestador = None
        self._alicuotaAplicable = None
        self._fechaIngresoImpuesto = None
        self._montoImpuesto = None
        self._impuestoComputable = None
        self._idPago = None
        self._cuitEntidadPago = None

    def get_values(self):
        values = [self.tipoComprobante, self.descripcion, self.identificacionComprobante,
                  self.fechaOperacion, self.montoMonedaOriginal, self.codigoMoneda,
                  self.tipoCambio, self.cuitPrestador, self.nifPrestador, self.nombrePrestador,
                  self.alicuotaAplicable, self.fechaIngresoImpuesto, self.montoImpuesto,
                  self.impuestoComputable, self.idPago, self.cuitEntidadPago]

        return values

    @property
    def tipoComprobante(self):
        return self._tipoComprobante

    @tipoComprobante.setter
    def tipoComprobante(self, tipoComprobante):
        self._tipoComprobante = self._fill_and_validate_len(tipoComprobante, 'tipoComprobante', 1)

    @property
    def descripcion(self):
        return self._descripcion

    @descripcion.setter
    def descripcion(self, descripcion):
        self._descripcion = self._fill_and_validate_len(descripcion, 'descripcion', 20, numeric=False)

    @property
    def identificacionComprobante(self):
        return self._identificacionComprobante

    @identificacionComprobante.setter
    def identificacionComprobante(self, identificacionComprobante):
        self._identificacionComprobante = self._fill_and_validate_len(identificacionComprobante,
                                                                      'identificacionComprobante', 20, numeric=False)

    @property
    def fechaOperacion(self):
        return self._fechaOperacion

    @fechaOperacion.setter
    def fechaOperacion(self, fechaOperacion):
        self._fechaOperacion = self._fill_and_validate_len(fechaOperacion, 'fechaOperacion', 8)

    @property
    def montoMonedaOriginal(self):
        return self._montoMonedaOriginal

    @montoMonedaOriginal.setter
    def montoMonedaOriginal(self, montoMonedaOriginal):
        self._montoMonedaOriginal = self._fill_and_validate_len(montoMonedaOriginal, 'montoMonedaOriginal', 15)

    @property
    def codigoMoneda(self):
        return self._codigoMoneda

    @codigoMoneda.setter
    def codigoMoneda(self, codigoMoneda):
        self._codigoMoneda = self._fill_and_validate_len(codigoMoneda, 'codigoMoneda', 3)

    @property
    def tipoCambio(self):
        return self._tipoCambio

    @tipoCambio.setter
    def tipoCambio(self, tipoCambio):
        self._tipoCambio = self._fill_and_validate_len(tipoCambio, 'tipoCambio', 10)

    @property
    def cuitPrestador(self):
        return self._cuitPrestador

    @cuitPrestador.setter
    def cuitPrestador(self, cuitPrestador):
        self._cuitPrestador = self._fill_and_validate_len(cuitPrestador, 'cuitPrestador', 11)

    @property
    def nifPrestador(self):
        return self._nifPrestador

    @nifPrestador.setter
    def nifPrestador(self, nifPrestador):
        self._nifPrestador = self._fill_and_validate_len(nifPrestador, 'nifPrestador', 20, numeric=False)

    @property
    def nombrePrestador(self):
        return self._nombrePrestador

    @nombrePrestador.setter
    def nombrePrestador(self, nombrePrestador):
        self._nombrePrestador = self._fill_and_validate_len(nombrePrestador, 'nombrePrestador', 30, numeric=False)

    @property
    def alicuotaAplicable(self):
        return self._alicuotaAplicable

    @alicuotaAplicable.setter
    def alicuotaAplicable(self, alicuotaAplicable):
        self._alicuotaAplicable = self._fill_and_validate_len(alicuotaAplicable, 'alicuotaAplicable', 4)

    @property
    def fechaIngresoImpuesto(self):
        return self._fechaIngresoImpuesto

    @fechaIngresoImpuesto.setter
    def fechaIngresoImpuesto(self, fechaIngresoImpuesto):
        self._fechaIngresoImpuesto = self._fill_and_validate_len(fechaIngresoImpuesto, 'fechaIngresoImpuesto', 8)

    @property
    def montoImpuesto(self):
        return self._montoImpuesto

    @montoImpuesto.setter
    def montoImpuesto(self, montoImpuesto):
        self._montoImpuesto = self._fill_and_validate_len(montoImpuesto, 'montoImpuesto', 15)

    @property
    def impuestoComputable(self):
        return self._impuestoComputable

    @impuestoComputable.setter
    def impuestoComputable(self, impuestoComputable):
        self._impuestoComputable = self._fill_and_validate_len(impuestoComputable, 'impuestoComputable', 15)

    @property
    def idPago(self):
        return self._idPago

    @idPago.setter
    def idPago(self, idPago):
        self._idPago = self._fill_and_validate_len(idPago, 'idPago', 20, numeric=False)

    @property
    def cuitEntidadPago(self):
        return self._cuitEntidadPago

    @cuitEntidadPago.setter
    def cuitEntidadPago(self, cuitEntidadPago):
        self._cuitEntidadPago = self._fill_and_validate_len(cuitEntidadPago, 'cuitEntidadPago', 11)


class SifereLine(PresentationLine):
    __slots__ = ['_jurisdiccion', '_cuit', '_fecha',
                 '_puntoDeVenta', '_tipo', '_letra',
                 '_importe']

    def __init__(self):
        self._jurisdiccion = None
        self._cuit = None
        self._fecha = None
        self._puntoDeVenta = None
        self._tipo = None
        self._letra = None
        self._importe = None

    @property
    def jurisdiccion(self):
        return self._jurisdiccion

    @jurisdiccion.setter
    def jurisdiccion(self, jurisdiccion):
        self._jurisdiccion = self._fill_and_validate_len(jurisdiccion, 'jurisdiccion', 3)

    @property
    def cuit(self):
        return self._cuit

    @cuit.setter
    def cuit(self, cuit):
        self._cuit = self._fill_and_validate_len(cuit, 'cuit', 13)

    @property
    def fecha(self):
        return self._fecha

    @fecha.setter
    def fecha(self, fecha):
        self._fecha = self._fill_and_validate_len(fecha, 'fecha', 10)

    @property
    def puntoDeVenta(self):
        return self._puntoDeVenta

    @puntoDeVenta.setter
    def puntoDeVenta(self, puntoDeVenta):
        self._puntoDeVenta = self._fill_and_validate_len(puntoDeVenta, 'puntoDeVenta', 4)

    @property
    def tipo(self):
        return self._tipo

    @tipo.setter
    def tipo(self, tipo):
        self._tipo = self._fill_and_validate_len(tipo, 'tipo', 1)

    @property
    def letra(self):
        return self._letra

    @letra.setter
    def letra(self, letra):
        self._letra = self._fill_and_validate_len(letra, 'letra', 1)

    @property
    def importe(self):
        return self._importe

    @importe.setter
    def importe(self, importe):
        self._importe = self._fill_and_validate_len(importe, 'importe', 11)


class SifereRetentionLine(SifereLine):
    __slots__ = ['_numeroBase', '_numeroComprobante']

    def __init__(self):
        super(SifereRetentionLine, self).__init__()
        self._numeroBase = None
        self._numeroComprobante = None

    @property
    def numeroBase(self):
        return self._numeroBase

    @numeroBase.setter
    def numeroBase(self, numeroBase):
        self._numeroBase = self._fill_and_validate_len(numeroBase, 'numeroBase', 20)

    @property
    def numeroComprobante(self):
        return self._numeroComprobante

    @numeroComprobante.setter
    def numeroComprobante(self, numeroComprobante):
        self._numeroComprobante = self._fill_and_validate_len(numeroComprobante, 'numeroComprobante', 16)

    def get_values(self):
        values = [self._jurisdiccion, self._cuit, self._fecha, self._puntoDeVenta,
                  self._numeroComprobante, self._tipo, self._letra, self._numeroBase,
                  self._importe]

        return values


class SiferePerceptionLine(SifereLine):
    __slots__ = ['_numeroComprobante']

    def __init__(self):
        super(SiferePerceptionLine, self).__init__()
        self._numeroComprobante = None

    @property
    def numeroComprobante(self):
        return self._numeroComprobante

    @numeroComprobante.setter
    def numeroComprobante(self, numeroComprobante):
        self._numeroComprobante = self._fill_and_validate_len(numeroComprobante, 'numeroComprobante', 8)

    def get_values(self):
        values = [self._jurisdiccion, self._cuit, self._fecha, self._puntoDeVenta,
                  self._numeroComprobante, self._tipo, self._letra, self._importe]

        return values


class SicoreLine(PresentationLine):
    __slots__ = []


class SicoreRetentionLine(SicoreLine):
    __slots__ = ['_codigoComprobante', '_fechaDocumento', '_referenciaDocumento',
                 '_importeDocumento', '_codigoImpuesto', '_codigoRegimen',
                 '_codigoOperacion', '_base', '_fecha', '_codigoCondicion',
                 '_retencionPracticadaSS', '_importe', '_porcentaje', '_fechaEmision',
                 '_codigoDocumento', '_cuit', '_numeroCertificado']

    def __init__(self):
        super(SicoreRetentionLine, self).__init__()
        self._codigoComprobante = None
        self._fechaDocumento = None
        self._referenciaDocumento = None
        self._importeDocumento = None
        self._codigoImpuesto = None
        self._codigoRegimen = None
        self._codigoOperacion = None
        self._base = None
        self._fecha = None
        self._codigoCondicion = None
        self._retencionPracticadaSS = None
        self._importe = None
        self._porcentaje = None
        self._fechaEmision = None
        self._codigoDocumento = None
        self._cuit = None
        self._numeroCertificado = None

    @property
    def codigoComprobante(self):
        return self._codigoComprobante

    @codigoComprobante.setter
    def codigoComprobante(self, codigoComprobante):
        self._codigoComprobante = self._fill_and_validate_len(codigoComprobante, 'codigoComprobante', 2)

    @property
    def fechaDocumento(self):
        return self._fechaDocumento

    @fechaDocumento.setter
    def fechaDocumento(self, fechaDocumento):
        self._fechaDocumento = self._fill_and_validate_len(fechaDocumento, 'fechaDocumento', 10)

    @property
    def referenciaDocumento(self):
        return self._referenciaDocumento

    @referenciaDocumento.setter
    def referenciaDocumento(self, referenciaDocumento):
        self._referenciaDocumento = self._fill_and_validate_len(referenciaDocumento, 'referenciaDocumento', 16)

    @property
    def importe(self):
        return self._importe

    @importe.setter
    def importe(self, importe):
        self._importe = self._fill_and_validate_len(importe, 'importe', 14)

    @property
    def codigoImpuesto(self):
        return self._codigoImpuesto

    @codigoImpuesto.setter
    def codigoImpuesto(self, codigoImpuesto):
        self._codigoImpuesto = self._fill_and_validate_len(codigoImpuesto, 'codigoImpuesto', 3)

    @property
    def codigoRegimen(self):
        return self._codigoRegimen

    @codigoRegimen.setter
    def codigoRegimen(self, codigoRegimen):
        self._codigoRegimen = self._fill_and_validate_len(codigoRegimen, 'codigoRegimen', 3)

    @property
    def codigoOperacion(self):
        return self._codigoOperacion

    @codigoOperacion.setter
    def codigoOperacion(self, codigoOperacion):
        self._codigoOperacion = self._fill_and_validate_len(codigoOperacion, 'codigoOperacion', 1)

    @property
    def base(self):
        return self._base

    @base.setter
    def base(self, base):
        self._base = self._fill_and_validate_len(base, 'base', 14)

    @property
    def fecha(self):
        return self._fecha

    @fecha.setter
    def fecha(self, fecha):
        self._fecha = self._fill_and_validate_len(fecha, 'fecha', 10)

    @property
    def codigoCondicion(self):
        return self._codigoCondicion

    @codigoCondicion.setter
    def codigoCondicion(self, codigoCondicion):
        self._codigoCondicion = self._fill_and_validate_len(codigoCondicion, 'codigoCondicion', 2)

    @property
    def retencionPracticadaSS(self):
        return self._retencionPracticadaSS

    @retencionPracticadaSS.setter
    def retencionPracticadaSS(self, retencionPracticadaSS):
        self._retencionPracticadaSS = self._fill_and_validate_len(retencionPracticadaSS, 'retencionPracticadaSS', 1)

    @property
    def importeDocumento(self):
        return self._importeDocumento

    @importeDocumento.setter
    def importeDocumento(self, importeDocumento):
        self._importeDocumento = self._fill_and_validate_len(importeDocumento, 'importeDocumento', 16)

    @property
    def porcentaje(self):
        return self._porcentaje

    @porcentaje.setter
    def porcentaje(self, porcentaje):
        self._porcentaje = self._fill_and_validate_len(porcentaje, 'porcentaje', 6)

    @property
    def fechaEmision(self):
        return self._fechaEmision

    @fechaEmision.setter
    def fechaEmision(self, fechaEmision):
        self._fechaEmision = self._fill_and_validate_len(fechaEmision, 'fechaEmision', 10)

    @property
    def codigoDocumento(self):
        return self._codigoDocumento

    @codigoDocumento.setter
    def codigoDocumento(self, codigoDocumento):
        self._codigoDocumento = self._fill_and_validate_len(codigoDocumento, 'codigoDocumento', 2)

    @property
    def cuit(self):
        return self._cuit

    @cuit.setter
    def cuit(self, cuit):
        self._cuit = self._fill_and_validate_len(cuit, 'cuit', 20)

    @property
    def numeroCertificado(self):
        return self._numeroCertificado

    @numeroCertificado.setter
    def numeroCertificado(self, numeroCertificado):
        self._numeroCertificado = self._fill_and_validate_len(numeroCertificado, 'numeroCertificado', 14)

    def get_values(self):
        values = [self._codigoComprobante,
                  self._fechaDocumento,
                  self._referenciaDocumento,
                  self._importeDocumento,
                  self._codigoImpuesto,
                  self._codigoRegimen,
                  self._codigoOperacion,
                  self._base,
                  self._fecha,
                  self._codigoCondicion,
                  self._retencionPracticadaSS,
                  self._importe,
                  self._porcentaje,
                  self._fechaEmision,
                  self._codigoDocumento,
                  self._cuit,
                  self._numeroCertificado, ]

        return values



import unittest
import sys
import random
sys.path.append("../")
import banks

class TestContribuyentes(unittest.TestCase):

    def setUp(self):
        self.banks_list = banks.Banks.get_banks_list()
        self.values = banks.Banks.get_values(self.banks_list)
        
    def test_check_values_types(self):
        
        values = []
        values.append(self.values[0])
        values.append(self.values[-1])
        values.append(random.choice(self.values))
        
        for bank in values:
            
            int(bank['cuit']) 
            int(bank['code'])
    
    def test_check_invalid_values_types(self):
        
        values = []
        values.append(self.values[0])
        values.append(self.values[-1])
        values.append(random.choice(self.values))
        
        for bank in values:
            
            int(bank['cuit']) 
            int(bank['code'])
            self.assertRaises(ValueError, lambda: int(bank['name']))

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
import config
import json
import urllib2

class Contributor:
    
    def __init__(self):
        self.name = None
        self.document_type = None
        self.document_number = None
        self.street = None
        self.postal_code = None
        self.city = None
        self.state_code = None
        
    
    def set_partner_data(self, document_number=None):
        """ 
        Completa los datos del contribuyente con los datos del API Rest de AFIP
        :param document_number: Numero de documento con el cual se consultara
        """
        
        if document_number: self.document_number = document_number 
        
        if self.document_number:
            data = self.get_partner_data(self.document_number).get('data')
            fiscal_location = None
            
            if data:

                self.name = data.get('nombre')
                fiscal_location = data.get('domicilioFiscal')
            
            if fiscal_location:
                self.postal_code = fiscal_location.get('codPostal')      
                self.city = fiscal_location.get('localidad')
                self.street = fiscal_location.get('direccion')           
                self.state_code = fiscal_location.get('idProvincia')
        
    @classmethod
    def is_valid_cuit(cls, cuit):
        """
        :param cuit: Cuit a validar
        :return: Falso si no es valido
        """
        
        try:
            int(cuit)
        except ValueError:
            return False
        
        cuit = str(cuit)
        
        if len(cuit) != 11:
            return False
        
        l=[5,4,3,2,7,6,5,4,3,2]

        var1=0
        for i in range(10):
            var1=var1+int(cuit[i])*l[i]
        var3=11-var1%11

        if var3==11: var3=0
        if var3==10: var3=9
        if var3 == int(cuit[10]):

            return True

        return False
    
    @classmethod
    def get_partner_data(cls, document_number, url=None):
        """
        Obtiene los datos del API Rest de AFIP
        :param document_number: Cuit a consultar
        :param url: Url de donde se realizara la consulta
        :raises: ValueError si la url es invalida o no se puede obtener un JSON.
        :returns: JSON con la respuesta para ese contribuyente.
        """
        
        url = url+str(document_number) if url else config.URL_PADRON+str(document_number)
        try:
            res = json.load(urllib2.urlopen(url))
        except ValueError:
            raise
        
        return res
        
        


# -*- coding: utf-8 -*-
import pytz
from datetime import datetime
import time
from lxml import etree
from M2Crypto import BIO, SMIME


class AccessRequerimentToken(object):
    
    def __init__(self, service, tz=pytz.utc):
        self._service = service
        self._timezone = tz

    def _create_tra(self):
        '''
        uniqueId: Entero de 32 bits sin signo que junto con generationTime identifica el
        requerimiento.
        
        generationTime: Momento en que fue generado el requerimiento. La tolerancia de aceptacion
        sera de hasta 24 horas previas al requerimiento de acceso
        
        expirationTime: Momento en el que expira la solicitud. La tolerancia de aceptacion sera de
        hasta 24 horas posteriores al requerimiento de acceso.  
        
        service: Identificacion del WSN para el cual se solicita el TA.
        
        :return: Ticket de Requerimiento de Acceso
        '''
        
        ticketRequest = etree.Element('loginTicketRequest')
        ticketRequest.set('version', '1.0')
        header = etree.SubElement(ticketRequest, 'header')

        #uniqueId
        uniqueid = etree.SubElement(header, 'uniqueId')
        #Traemos la fecha actual
        timestamp = int(time.mktime(datetime.now().timetuple()))
        uniqueid.text = str(timestamp)
        
        #generationTime
        # Generamos el token con media hora atrasada
        tsgen = datetime.fromtimestamp(timestamp-1800)
        tsgen = pytz.utc.localize(tsgen).astimezone(self.timezone)
        gentime = etree.SubElement(header, 'generationTime')
        gentime.text = tsgen.isoformat()    

        # expirationTime
        # El token vence 4.5 horas despues
        tsexp = datetime.fromtimestamp(timestamp+14400)
        tsexp = pytz.utc.localize(tsexp).astimezone(self.timezone)
        exptime = etree.SubElement(header, 'expirationTime')
        exptime.text = tsexp.isoformat()
        exptime.tail = None

        #service
        serv = etree.SubElement(ticketRequest, 'service')
        serv.text = self.service

        return etree.tostring(ticketRequest)

    def sign_tra(self, private_key, certificate):

        smime = SMIME.SMIME()
        ks = BIO.MemoryBuffer(private_key.encode('ascii'))
        cs = BIO.MemoryBuffer(certificate.encode('ascii'))
        bf = BIO.MemoryBuffer(str(self._create_tra()))
        out = BIO.MemoryBuffer()

        try:
            smime.load_key_bio(ks, cs)
        except Exception:
            raise Exception('Error en el formato del certificado o clave privada')

        sbf = smime.sign(bf)
        smime.write(out, sbf)

        head, body, end = out.read().split('\n\n')

        return body
    
    @property
    def service(self):
        return self._service

    @property
    def timezone(self):
        return self._timezone

class AccessToken(object):
    
    ''' Ejemplo de token:
    
    <?xml version="1.0" encoding="UTF8"?>
    <loginTicketResponse version="1.0">
    <header>
        <source>cn=wsaa,o=afip,c=ar,serialNumber=CUIT 33693450239</source>
        <destination>cn=srv1,ou=facturacion,o=empresa s.a.,c=ar,serialNumber=CUIT 30123456789</destination>
        <uniqueId>383953094</uniqueId>
        <generationTime>20011231T12:00:0203:00</generationTime>
        <expirationTime>20020101T00:00:0203:00</expirationTime>
    </header>
    <credentials>
        <token>cES0SSuWIIPlfe5/dLtb0Qeg2jQuvYuuSEDOrz+w2EnAQiEeS86gzYf7ehiU3UaYit5FRb9z/3zq</token>
        <sign>a6QSSZBgLf0TTcktSNteeSg3qXsMVjo/F5py/Gtw7xucTrUWbsrVCdIoGE8Cm1bixpuVPlr58k6n</sign>
    </credentials>
    </loginTicketResponse>
    '''
    
    def __init__(self):
        #Header
        self.source = None
        self.destination = None
        self.uniqueId = None
        self.generation_time = None
        self.expiration_time = None

        #Credentials
        self.token = None
        self.sign = None
        
    def create_token_from_login(self, login_fault):
        '''
        Asigna los atributos al token desde el login
        :param login_fault: XML con el login autorizado 
        '''
        
        login_fault_tree = etree.fromstring(str(login_fault))
        
        try:
            
            self.source = login_fault_tree.find('header/source').text
            self.destination = login_fault_tree.find('header/destination').text
            self.generation_time = login_fault_tree.find('header/generationTime').text
            self.expiration_time = login_fault_tree.find('header/expirationTime').text
            
            self.token = login_fault_tree.find('credentials/token').text
            self.sign = login_fault_tree.find('credentials/sign').text
        
        except AttributeError: raise AttributeError("Error al generar el token de acceso")
        


import urllib2
from BeautifulSoup import BeautifulSoup
from unidecode import unidecode
from contributor import Contributor

class Banks:
             
    @staticmethod
    def get_banks_list():
        ''' 
        Obtiene la lista de bancos desde AFIP utilizando
        la libreria BeautifulSoup para webscraping
        '''
        
        url = 'http://www.afip.gov.ar/genericos/emisorasGarantias/formularioCompa%C3%B1ias.asp?completo=1&ent=3'
        req = urllib2.Request(url)
        f = urllib2.urlopen(req)
        soup = BeautifulSoup(f)
        
        table = soup.find('table', attrs={"class": "contenido"})
        
        banks = []
        for row in table.findAll('tr')[2:]:
            
            banks.append([td.getText() for td in row.findAll('td') if td.text])
                        
        return banks
    
    @staticmethod
    def get_values(banks_list):
        '''
        :param banks_list: Lista de bancos.
        :return: Lista de diccionarios con los valores de cada banco
        '''
        
        values = []
        for bank_list in banks_list:
            
            bank_values = {}
            
            if bank_list:    
                
                #Validamos el cuit del banco para comprobar existencia
                if Contributor.is_valid_cuit(bank_list[0]):
            
                    bank_values['cuit'] = bank_list[0]
                    bank_values['code'] = bank_list[1]
                    bank_values['name'] = unidecode(bank_list[2])
                    values.append(bank_values)
            
        return values
                
            